/**
 * @author luc
 */
package net.urlrewriter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.MatchResult;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

import net.urlrewriter.utilities.IContextFacade;
import net.urlrewriter.utilities.NameValueCollection;

/**
 * @author luc
 *
 */
public class RewriteContext {

	
	/**
	 * Engine 
	 */
	private RewriterEngine mEngine;
	
	
	/**
	 * Properties which contains request headers, cookies and server variables. 
	 */
	private NameValueCollection mProperties;
	
	/**
	 * Request headers 
	 */
	private NameValueCollection mHeaders;
	
	/**
	 * Request cookies 
	 */
	private List<Cookie> mCookies;
	
	/**
	 * Response status code 
	 */
	private int mHttpStatusCode = HttpServletResponse.SC_OK;
	
	/**
	 * URL 
	 */
	private String mLocation;
	
	/**
	 * Request HTTP Method 
	 */
	private String mHttpMethod;
	
	/**
	 * Current Engine Processing state 
	 */
	private RewriteProcessing mProcessing;
	
	/**
	 * Last match 
	 */
	private MatchResult mLastMatch;
	
	
	/**
	 * Context facade 
	 */
	private IContextFacade mContextFacade;
	

	/**
	 * 
	 */
	public RewriteContext(RewriterEngine pEngine, IContextFacade pFacade) {
		super();
		mEngine = pEngine;
		mLocation = pFacade.getRawUrl();
		mHttpMethod = pFacade.getHttpMethod();
		
		mProperties = new NameValueCollection();
		for (String oKey : pFacade.getHeaders().keySet()) {
			mProperties.add(oKey, pFacade.getHeaders().get(oKey));
		}
		Cookie[] oCookies = pFacade.getCookies();
		if (oCookies != null) {
			for (int i = 0; i < oCookies.length; i++) {
				Cookie oCookie = oCookies[i];
				mProperties.add(oCookie.getName(), oCookie.getValue());
			}
		}
		if (pFacade.getServerVariables() != null) {
			for (String oKey : pFacade.getServerVariables().keySet()) {
				mProperties.add(oKey, pFacade.getServerVariables().get(oKey));
			}
		}
		mHeaders = new NameValueCollection();
		mCookies = new ArrayList<Cookie>();
		mContextFacade = pFacade;
	}

	/**
	 * @return Returns the httpStatusCode.
	 */
	public int getHttpStatusCode() {
		return mHttpStatusCode;
	}


	/**
	 * @param pHttpStatusCode The httpStatusCode to set.
	 */
	public void setHttpStatusCode(int pHttpStatusCode) {
		mHttpStatusCode = pHttpStatusCode;
	}


	/**
	 * @return Returns the lastMatch.
	 */
	public MatchResult getLastMatch() {
		return mLastMatch;
	}


	/**
	 * @param pLastMatch The lastMatch to set.
	 */
	public void setLastMatch(MatchResult pLastMatch) {
		mLastMatch = pLastMatch;
	}


	/**
	 * @return Returns the location.
	 */
	public String getLocation() {
		return mLocation;
	}


	/**
	 * @param pLocation The location to set.
	 */
	public void setLocation(String pLocation) {
		mLocation = pLocation;
	}


	/**
	 * @return Returns the processing.
	 */
	public RewriteProcessing getProcessing() {
		return mProcessing;
	}


	/**
	 * @param pProcessing The processing to set.
	 */
	public void setProcessing(RewriteProcessing pProcessing) {
		mProcessing = pProcessing;
	}


	/**
	 * @return Returns the cookies.
	 */
	public List<Cookie> getCookies() {
		return mCookies;
	}


	/**
	 * @return Returns the headers.
	 */
	public NameValueCollection getHeaders() {
		return mHeaders;
	}


	/**
	 * @return Returns the httpMethod.
	 */
	public String getHttpMethod() {
		return mHttpMethod;
	}


	/**
	 * @return Returns the properties.
	 */
	public NameValueCollection getProperties() {
		return mProperties;
	}
	
	/**
	* Expands the given input using the last match, properties, maps and transforms.
	* @param input The input to expand.
	* @return The expanded form of the input.
	*/
	public String expand(String pInput) throws IOException {
		return mEngine.expand(this, pInput);
	}

	/**
	 * Resolves the location to an absolute reference.
	 * @param pLocation The application-referenced location.
	 * @return The absolute location.
	 */
	public String resolveLocation(String pLocation) {
		return mEngine.resolveLocation(pLocation, mContextFacade);
	}
	
	public String mapPath(String pUrl) {
		return mContextFacade.mapPath(pUrl);
	}
	

}
