/**
 * @author luc
 */
package net.urlrewriter;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.urlrewriter.errors.IRewriteErrorHandler;
import net.urlrewriter.utilities.IContextFacade;
import net.urlrewriter.utilities.NameValueCollection;

/**
 * @author luc
 *
 */
public class RewriteResponseWrapper extends HttpServletResponseWrapper {

	private Log mLogger = LogFactory.getLog(RewriteResponseWrapper.class);
	
	private RewriterEngine mEngine;
	private HttpServletRequest mRequest;
	private HttpServletResponse mResponse;
	private ServletContext mServletContext;
	private URI mRequestURI;
	
	private Map<String, String> mURLEncodedCache = new HashMap<String, String>();
	
	/**
	 * @param pResponse
	 */
	public RewriteResponseWrapper(HttpServletResponse pResponse, HttpServletRequest pRequest, RewriterEngine pEngine, ServletContext pContext) {
		super(pResponse);
		try {
			mRequestURI = new URI(pRequest.getRequestURI());
			mEngine = pEngine;
			mRequest = pRequest;
			mResponse = pResponse;
		} catch (Exception e) {
			mLogger.fatal("Error parsing URI \"" + pRequest.getRequestURI() + "\".", e);
		}
	}
	
	@Override
	public String encodeURL(String pUrl) {
		return processUrl(pUrl);
	}

	@Override
	public String encodeRedirectURL(String pUrl) {
		return processUrl(pUrl);
	}
	
	@Override
	public String encodeRedirectUrl(String pUrl) {
		return processUrl(pUrl);
	}
	
	@Override
	public String encodeUrl(String pUrl) {
		return processUrl(pUrl);
	}
	
	private String processUrl(String pUrl) {
		String oCached = mURLEncodedCache.get(pUrl);
		if (oCached != null) return oCached;
		String oResult = pUrl;
		try {
			URI oURI = new URI(createFullUrl(pUrl));
			InlineContextFacade oFacade = new InlineContextFacade(mServletContext, mRequest.getContextPath(), mRequest.getMethod(), pUrl, oURI);
			if (mEngine.rewrite(oFacade)) {
				oResult = oFacade.getResult();
			}
		} catch (Exception e) {
			mLogger.fatal("Error encoding url \"" + pUrl + "\".", e);
		}
		if (mLogger.isInfoEnabled()) mLogger.info("Encoding url \"" + pUrl + "\" to \"" + oResult + "\".");
		mURLEncodedCache.put(pUrl, oResult);
		return oResult;
	}
	
	private String createFullUrl(String pRawUrl) {
		String oPattern = "%1$s://%2$s%3$s";
		if (mRequest.getServerPort() != 80) {
			oPattern = "%1$s://%2$s:%4$s%3$s";
		}
		return String.format(oPattern, new Object[] {mRequest.getScheme(), mRequest.getServerName(), mRequest.getContextPath() + "/" + pRawUrl, mRequest.getServerPort()});
	}
	
	class InlineContextFacade implements IContextFacade {

		private ServletContext mContext;
		private String mAppPath;
		private String mMethod;
		private String mRawUrl;
		private URI mRequestUrl;
		private int mStatusCode;
		private List<Cookie> mCookies = new ArrayList<Cookie>();
		private NameValueCollection mHeaders = new NameValueCollection();
		private NameValueCollection mVariables = new NameValueCollection();
		private Map<String, Object> mItems = new Hashtable<String, Object>();
		
		private String mResult;
		
		public InlineContextFacade(ServletContext pContext, String pAppPath, String pMethod, String pRawUrl, URI pRequestUrl) {
			mContext = pContext;
			mAppPath = pAppPath;
			mMethod = pMethod;
			mRawUrl = pRawUrl;
			mRequestUrl = pRequestUrl;
		}
		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#appendCookie(javax.servlet.http.Cookie)
		 */
		public void appendCookie(Cookie pCookie) {
			mCookies.add(pCookie);
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#appendHeader(java.lang.String, java.lang.String)
		 */
		public void appendHeader(String pName, String pValue) {
			mHeaders.add(pName, pValue);
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getApplicationPath()
		 */
		public String getApplicationPath() {
			return mAppPath;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getCookies()
		 */
		public Cookie[] getCookies() {
			return mCookies.toArray(new Cookie[0]);
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getHeaders()
		 */
		public NameValueCollection getHeaders() {
			return mHeaders;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getHttpMethod()
		 */
		public String getHttpMethod() {
			return mMethod;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getItem(java.lang.String)
		 */
		public Object getItem(String pKey) {
			return mItems.get(pKey);
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getRawUrl()
		 */
		public String getRawUrl() {
			return mRawUrl;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getRequestUrl()
		 */
		public URI getRequestUrl() {
			return mRequestUrl;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#getServerVariables()
		 */
		public NameValueCollection getServerVariables() {
			return mVariables;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#handleError(net.urlrewriter.errors.IRewriteErrorHandler)
		 */
		public void handleError(IRewriteErrorHandler pHandler) {
			pHandler.handleError(this);
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#mapPath(java.lang.String)
		 */
		public String mapPath(String pUrl) {
			return mContext.getRealPath(pUrl);
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#rewritePath(java.lang.String)
		 */
		public void rewritePath(String pUrl) {
			mResult = pUrl;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#setItem(java.lang.String, java.lang.Object)
		 */
		public void setItem(String pKey, Object pItem) {
			mItems.put(pKey, pItem);
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#setRedirectLocation(java.lang.String)
		 */
		public void setRedirectLocation(String pUrl) {
			mResult = pUrl;
		}

		/* (non-Javadoc)
		 * @see net.urlrewriter.utilities.IContextFacade#setStatusCode(int)
		 */
		public void setStatusCode(int pCode) {
			mStatusCode = pCode;
		}
		
		public String getResult() {
			return mResult;
		}
		
	}
}
