/**
 * @author luc
 */
package net.urlrewriter;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URI;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.Cookie;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.urlrewriter.actions.IRewriteAction;
import net.urlrewriter.conditions.IRewriteCondition;
import net.urlrewriter.configuration.RewriterConfiguration;
import net.urlrewriter.errors.IRewriteErrorHandler;
import net.urlrewriter.utilities.IContextFacade;
import net.urlrewriter.utilities.Messages;

/**
 * @author luc
 *
 */
public class RewriterEngine {
	
	private static final String STRING_EMPTY = "";
	private static final int MAX_RESTART = 10; // Controls the number of restarts so we don't get into an infinite loop
	
	private RewriterConfiguration mConfiguration;
	
	private Log mLogger = LogFactory.getLog(RewriterEngine.class);
	
	/**
	 * 
	 */
	public RewriterEngine(RewriterConfiguration pConfiguration) {
		mConfiguration = pConfiguration;
	}
	
	public String resolveLocation(String pLocation, IContextFacade pContextFacade) {
		String oAppPath = pContextFacade.getApplicationPath();

		if (oAppPath.length() > 1)
		{
			oAppPath += "/";
		}
		String oResolvedLocation = pLocation.replaceAll("\\~/", oAppPath);
		if (mLogger.isInfoEnabled()) mLogger.info("Location \"" + pLocation + "\" resolved to \"" + oResolvedLocation + "\".");
		return oResolvedLocation;
	}
	
	public boolean rewrite(IContextFacade pContextFacade) throws RewriteException {
		String oOriginalUrl = pContextFacade.getRawUrl();

		// Create the context
		RewriteContext oContext = new RewriteContext(this, pContextFacade);

		// Process each rule.
		processRules(oContext);
		
		// Append any headers defined.
		appendHeaders(oContext, pContextFacade);

		// Append any cookies defined.
		appendCookies(oContext, pContextFacade);

		boolean oResponseCommitted = false;
		// Rewrite the path if the location has changed.
		pContextFacade.setStatusCode(oContext.getHttpStatusCode());
		if ((!oOriginalUrl.equals(oContext.getLocation())) && (oContext.getHttpStatusCode() < 400)) {
			if (oContext.getHttpStatusCode() < 300) {
				// Successful status if less than 300
				//_configuration.Logger.Info(MessageProvider.FormatString(Message.RewritingXtoY, 
				//	ContextFacade.GetRawUrl(), context.Location));
				if (mLogger.isDebugEnabled()) mLogger.debug("Rewriting \"" + pContextFacade.getRawUrl() + "\" to \"" + oContext.getLocation() + "\".");
				// Verify that the url exists on this server.
				verifyResultExists(oContext, pContextFacade);

				pContextFacade.rewritePath(oContext.getLocation());
			} else {
				oResponseCommitted = true;
				// Redirection
				//_configuration.Logger.Info(MessageProvider.FormatString(Message.RedirectingXtoY,
				//	ContextFacade.GetRawUrl(), context.Location));
				if (mLogger.isDebugEnabled()) mLogger.debug("Redirecting from \"" + pContextFacade.getRawUrl() + "\" to \"" + oContext.getLocation() + "\" (" + oContext.getHttpStatusCode() + ").");
				pContextFacade.setRedirectLocation(oContext.getLocation());
			}
		} else if (oContext.getHttpStatusCode() >= 400) {
			oResponseCommitted = true;
			handleError(oContext, pContextFacade);
		}

		// Sets the context items.
		setContextItems(oContext, pContextFacade);
		
		// return HttpStatusCode
		return oResponseCommitted;
	}
	
	protected void processRules(RewriteContext pContext) throws RewriteException {
		List oRewriteRules = mConfiguration.getRules();
		int oRestarts = 0;
		RewriteProcessing oProcessing = null;
		for (int i = 0; i < oRewriteRules.size(); i++) {
			// If the rule is conditional, ensure the conditions are met.
			Object oRule = oRewriteRules.get(i);
			if (oRule instanceof IRewriteCondition) {
				IRewriteCondition oCondition = (IRewriteCondition) oRule;
				if (!oCondition.isMatch(pContext)) continue;
			}
			// Execute the action.
			IRewriteAction oAction = (IRewriteAction) oRewriteRules.get(i);
			oProcessing = oAction.execute(pContext);
		
			// If the action is Stop, then break out of the processing loop
			if (oProcessing == RewriteProcessing.StopProcessing) {
				//_configuration.Logger.Debug(MessageProvider.FormatString(Message.StoppingBecauseOfRule));
				if (mLogger.isDebugEnabled()) mLogger.debug("Stopping because of rule.");
				break;
			} else if (oProcessing == RewriteProcessing.RestartProcessing) {
				//_configuration.Logger.Debug(MessageProvider.FormatString(Message.RestartingBecauseOfRule));
				if (mLogger.isDebugEnabled()) mLogger.debug("Restarting because of rule.");
				// Restart from the first rule.
				i = 0;
				if (++oRestarts > MAX_RESTART) {
					throw new RewriteException(String.format(Messages.TOO_MANY_RESTART));
				}
			}
		}
	}
	
	protected void appendHeaders(RewriteContext pContext, IContextFacade pContextFacade) {
		for (String oKey : pContext.getHeaders().keySet()) {
			pContextFacade.appendHeader(oKey, pContext.getHeaders().get(oKey));
		}
	}
	
	protected void appendCookies(RewriteContext pContext, IContextFacade pContextFacade) {
		for (Cookie oCookie : pContext.getCookies()) {
			pContextFacade.appendCookie(oCookie);
		}
	}
	
	protected void verifyResultExists(RewriteContext pContext, IContextFacade pContextFacade) {
		if (!pContextFacade.getRawUrl().equalsIgnoreCase(pContext.getLocation()) && pContext.getHttpStatusCode() < 300) {
			//URI oUri = new URI(pContextFacade.getRequestUrl(), pContext.getLocation());
			mLogger.debug("RequestURL:" + pContextFacade.getRequestUrl().toString() + ", location:" + pContext.getLocation());
			URI oURI = URI.create(pContext.getLocation());
			if (oURI.getHost() == pContextFacade.getRequestUrl().getHost()) {
				String oFilename = pContextFacade.mapPath(oURI.toString());
				if (! new File(oFilename).exists()) {
					//_configuration.Logger.Debug(MessageProvider.FormatString(Message.ResultNotFound, filename));
					mLogger.fatal("Result not found.");
					pContext.setHttpStatusCode(404);
				}
			}
		}
	}
	
	protected void handleError(RewriteContext pContext, IContextFacade pContextFacade) throws RewriteException {
	 	// Return the status code.
		pContextFacade.setStatusCode(pContext.getHttpStatusCode());
		// Get the error handler if there is one.
		IRewriteErrorHandler oHandler = mConfiguration.getErrorHandlers().get(pContext.getHttpStatusCode());
		if (oHandler != null) {
			try {
				//_configuration.Logger.Debug(MessageProvider.FormatString(Message.CallingErrorHandler));

				// Execute the error handler.
				pContextFacade.handleError(oHandler);
			} catch (Exception e) {
				//_configuration.Logger.Fatal(exc.Message, exc);
				mLogger.fatal("Couldn't handle error.", e);
				throw new RewriteException("Couldn't handle error.");
			}
		} else {
			//throw new HttpException((int)context.StatusCode, context.StatusCode.ToString());
			throw new RewriteException("No handler to process error for status code " + pContext.getHttpStatusCode());
		}
	}
	
	protected void setContextItems(RewriteContext pContext, IContextFacade pContextFacade) {
		//pContextFacade.setItem(CONTEXT_QUERYSTRING, new Uri(ContextFacade.GetRequestUrl(), context.Location).Query.Replace("?", ""));
		//pContextFacade.setItem(CONTEXT_ORIGINALQUERYSTRING, new Uri(ContextFacade.GetRequestUrl(), ContextFacade.GetRawUrl()).Query.Replace("?", ""));
	}
	
	public String expand(RewriteContext pContext, String pInput) throws IOException {
		/* replacement :- $n
		 * |	${[a-zA-Z0-9\-]+}
		 * |	${fn( <replacement> )}
		 * |	${<replacement-or-id>:<replacement-or-value>:<replacement-or-value>}
		 * 
		 * replacement-or-id :- <replacement> | <id>
		 * replacement-or-value :- <replacement> | <value>
		 */

		/* $1 - regex replacement
		 * ${propertyname}
		 * ${map-name:value}				map-name is replacement, value is replacement
		 * ${map-name:value|default-value}	map-name is replacement, value is replacement, default-value is replacement
		 * ${fn(value)}						value is replacement
		 */

		StringReader oReader = new StringReader(pInput);
		StringWriter oWriter = new StringWriter();			
		char c = (char) oReader.read();
		while (c != (char)65535)
		{
			if ((char)c == '$')
			{
				oWriter.write(reduce(pContext, oReader));
			}
			else
			{
				oWriter.write(c);
			}

			c = (char) oReader.read();
		}
		
		return oWriter.toString();
	
	}
	
	private String reduce(RewriteContext pContext, StringReader pReader) throws IOException {
		String oResult = null;
		char c = (char) pReader.read();
		if (Character.isDigit(c))
		{
			String oNum = Character.toString(c);
			pReader.mark(1);
			c = (char) pReader.read();
			if (Character.isDigit(c)) {
				pReader.reset();
				c = (char) pReader.read();
				oNum += Character.toString(c);
			} else {
				pReader.reset();
			}
			if (pContext.getLastMatch() != null) {
				//Group group = pContext.getLastMatch().Groups[Integer.parseInt(num)];
				String oGroup = pContext.getLastMatch().group(Integer.parseInt(oNum));
				if (oGroup != null)
				{
					oResult = oGroup;
				}
				else
				{
					oResult = STRING_EMPTY;
				}
			}
			else
			{
				oResult = STRING_EMPTY;
			}
		}
		/*
		else if (ch == '<')
		{
			StringWriter writer = new StringWriter();

			ch = (char) pReader.read();
			while (ch != '>' && ch != (char)65535)
			{
				if (ch == '$')
				{
					writer.write(reduce(pContext, pReader));
				}
				else
				{
					writer.write(ch);
				}
				ch = (char) pReader.read();
			}

			String expr = writer.toString();
			if (pContext.getLastMatch() != null)
			{
				Group group = pContext.getLastMatch().Groups[expr];
				if (group != null)
				{
					oResult = group.Value;
				}
				else
				{
					oResult = STRING_EMPTY;
				}
			}
			else
			{
				oResult = STRING_EMPTY;
			}
		}	
		*/		
		else if (c == '{')
		{
			StringWriter oWriter = new StringWriter();
			boolean oIsMap = false;
			boolean oIsFunction = false;

			c = (char) pReader.read();
			while (c != '}' && c != (char)65535)
			{
				if (c == '$')
				{
					oWriter.write(reduce(pContext,  pReader));
				}
				else
				{
					if (c == ':') oIsMap = true;
					else if (c == '(') oIsFunction = true;
					oWriter.write(c);
				}
				c = (char) pReader.read();
			}

			String oExpr = oWriter.toString();
			if (oIsMap)
			{
				Pattern oPattern = Pattern.compile("^([^\\:]+)\\:([^\\|]*)(\\|(.+))?$");
				Matcher oMatcher = oPattern.matcher(oExpr);
				if (oMatcher.matches()) {
					String oMapName = oMatcher.group(1);
					String oMapArgument = oMatcher.group(2);
					String oMapDefault = oMatcher.group(4);
					oResult = mConfiguration.getTransformFactory().getTransform(oMapName).applyTransform(oMapArgument);
					if (oResult == null) {
						oResult = oMapDefault;
					}
				}
			} else if (oIsFunction) {
				Pattern oPattern = Pattern.compile("^([^\\(]+)\\(([^\\)]*)\\)$");
				Matcher oMatcher = oPattern.matcher(oExpr);
				if (oMatcher.matches()) {
					String oFunctionName = oMatcher.group(1);
					String oFunctionArgument = oMatcher.group(2);
					oResult = mConfiguration.getTransformFactory().getTransform(oFunctionName).applyTransform(oFunctionArgument);
				}
			} else {
				oResult = pContext.getProperties().get(oExpr);
			}
		} else {
			oResult = Character.toString(c);
		}

		return oResult;
	}

}
