/**
 * @author luc
 */
package net.urlrewriter.configuration;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.urlrewriter.errors.IRewriteErrorHandler;
import net.urlrewriter.logging.IRewriteLogger;
import net.urlrewriter.parsers.AddHeaderActionParser;
import net.urlrewriter.parsers.AddressConditionParser;
import net.urlrewriter.parsers.ConditionActionParser;
import net.urlrewriter.parsers.ExistsConditionParser;
import net.urlrewriter.parsers.ForbiddenActionParser;
import net.urlrewriter.parsers.GoneActionParser;
import net.urlrewriter.parsers.HeaderMatchConditionParser;
import net.urlrewriter.parsers.IfRedirectActionParser;
import net.urlrewriter.parsers.IfRewriteActionParser;
import net.urlrewriter.parsers.MethodConditionParser;
import net.urlrewriter.parsers.NotAllowedActionParser;
import net.urlrewriter.parsers.NotFoundActionParser;
import net.urlrewriter.parsers.NotImplementedActionParser;
import net.urlrewriter.parsers.PropertyMatchConditionParser;
import net.urlrewriter.parsers.RedirectActionParser;
import net.urlrewriter.parsers.RewriteActionParser;
import net.urlrewriter.parsers.SetCookieActionParser;
import net.urlrewriter.parsers.SetPropertyActionParser;
import net.urlrewriter.parsers.SetStatusActionParser;
import net.urlrewriter.parsers.UrlMatchConditionParser;
import net.urlrewriter.transforms.Base64DecodeTransform;
import net.urlrewriter.transforms.Base64Transform;
import net.urlrewriter.transforms.DecodeTransform;
import net.urlrewriter.transforms.EncodeTransform;
import net.urlrewriter.transforms.LowerTransform;
import net.urlrewriter.transforms.UpperTransform;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * @author luc
 *
 */
public class RewriterConfiguration {

	private TransformFactory mTransformFactory;
	private ActionParserFactory mActionParserFactory;
	private ConditionParserPipeline mConditionParserPipeline;
	private IRewriteLogger mLogger;
	private List mRules = new ArrayList();
	private Map<Integer, IRewriteErrorHandler> mErrorHandlers = new Hashtable<Integer, IRewriteErrorHandler>();
	
	/**
	 * 
	 */
	protected RewriterConfiguration() {
		super();
		
		mActionParserFactory = new ActionParserFactory();
		mActionParserFactory.addParser(new ConditionActionParser());
		mActionParserFactory.addParser(new AddHeaderActionParser());
		mActionParserFactory.addParser(new SetCookieActionParser());
		mActionParserFactory.addParser(new SetPropertyActionParser());
		mActionParserFactory.addParser(new RewriteActionParser());
		mActionParserFactory.addParser(new IfRewriteActionParser());
		mActionParserFactory.addParser(new RedirectActionParser());
		mActionParserFactory.addParser(new IfRedirectActionParser());
		mActionParserFactory.addParser(new SetStatusActionParser());
		mActionParserFactory.addParser(new ForbiddenActionParser());
		mActionParserFactory.addParser(new GoneActionParser());
		mActionParserFactory.addParser(new NotAllowedActionParser());
		mActionParserFactory.addParser(new NotFoundActionParser());
		mActionParserFactory.addParser(new NotImplementedActionParser());
		
		mConditionParserPipeline = new ConditionParserPipeline();
		mConditionParserPipeline.add(new AddressConditionParser());
		mConditionParserPipeline.add(new HeaderMatchConditionParser());
		mConditionParserPipeline.add(new MethodConditionParser());
		mConditionParserPipeline.add(new PropertyMatchConditionParser());
		mConditionParserPipeline.add(new ExistsConditionParser());
		mConditionParserPipeline.add(new UrlMatchConditionParser());
		
		mTransformFactory = new TransformFactory();
		mTransformFactory.addTransform(new DecodeTransform());
		mTransformFactory.addTransform(new EncodeTransform());
		mTransformFactory.addTransform(new LowerTransform());
		mTransformFactory.addTransform(new UpperTransform());
		mTransformFactory.addTransform(new Base64Transform());
		mTransformFactory.addTransform(new Base64DecodeTransform());
	}
	
	public static RewriterConfiguration create() {
		return new RewriterConfiguration();
	}
	
	public static RewriterConfiguration loadFromFile(String pFilename) throws ParserConfigurationException, IOException, SAXException, ConfigurationException {
		File f = new File(pFilename);
		if (f.canRead()) {
			Document oDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(f);
			return loadFromNode(oDoc.getDocumentElement());
		} else {
			throw new IOException("Invalid access or permission (READ) to configuration file \"" + pFilename + "\".");
		}
	}
	
	public static RewriterConfiguration loadFromNode(Node pNode) throws ConfigurationException {
		RewriterConfigurationReader oReader = new RewriterConfigurationReader();
		return oReader.read(pNode);
	}

	/**
	 * @return Returns the transformFactory.
	 */
	public TransformFactory getTransformFactory() {
		return mTransformFactory;
	}

	/**
	 * @param pTransformFactory The transformFactory to set.
	 */
	public void setTransformFactory(TransformFactory pTransformFactory) {
		mTransformFactory = pTransformFactory;
	}

	/**
	 * @return Returns the logger.
	 */
	public IRewriteLogger getLogger() {
		return mLogger;
	}

	/**
	 * @param pLogger The logger to set.
	 */
	public void setLogger(IRewriteLogger pLogger) {
		mLogger = pLogger;
	}

	/**
	 * @return Returns the actionParserFactory.
	 */
	public ActionParserFactory getActionParserFactory() {
		return mActionParserFactory;
	}

	/**
	 * @param pActionParserFactory The actionParserFactory to set.
	 */
	public void setActionParserFactory(ActionParserFactory pActionParserFactory) {
		mActionParserFactory = pActionParserFactory;
	}

	/**
	 * @return Returns the conditionParserPipeline.
	 */
	public ConditionParserPipeline getConditionParserPipeline() {
		return mConditionParserPipeline;
	}

	/**
	 * @param pConditionParserPipeline The conditionParserPipeline to set.
	 */
	public void setConditionParserPipeline(
			ConditionParserPipeline pConditionParserPipeline) {
		mConditionParserPipeline = pConditionParserPipeline;
	}

	/**
	 * @return Returns the rules.
	 */
	public List getRules() {
		return mRules;
	}

	/**
	 * @param pRules The rules to set.
	 */
	public void setRules(List pRules) {
		mRules = pRules;
	}

	/**
	 * @return Returns the errorHandlers.
	 */
	public Map<Integer, IRewriteErrorHandler> getErrorHandlers() {
		return mErrorHandlers;
	}

	/**
	 * @param pErrorHandlers The errorHandlers to set.
	 */
	public void setErrorHandlers(Map<Integer, IRewriteErrorHandler> pErrorHandlers) {
		mErrorHandlers = pErrorHandlers;
	}

	

}
