/**
 * @author luc
 */
package net.urlrewriter.configuration;

import java.util.List;

import net.urlrewriter.actions.IRewriteAction;
import net.urlrewriter.errors.DefaultErrorHandler;
import net.urlrewriter.errors.IRewriteErrorHandler;
import net.urlrewriter.logging.IRewriteLogger;
import net.urlrewriter.parsers.IRewriteActionParser;
import net.urlrewriter.parsers.IRewriteConditionParser;
import net.urlrewriter.parsers.ParserHelper;
import net.urlrewriter.transforms.StaticMappingTransform;
import net.urlrewriter.utilities.Constants;
import net.urlrewriter.utilities.Messages;
import net.urlrewriter.utilities.StringDictionnary;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author luc
 *
 */
public class RewriterConfigurationReader {

	private Log mLogger = LogFactory.getLog(RewriterConfigurationReader.class);
	/**
	 * 
	 */
	public RewriterConfigurationReader() {
		super();
	}
	
	public RewriterConfiguration read(Node pNode) throws ConfigurationException {
		RewriterConfiguration oConfig = new RewriterConfiguration();
		NodeList oChildren = pNode.getChildNodes();
		Node oChild;
		for (int i = 0; i < oChildren.getLength(); i++) {
			oChild = oChildren.item(i);
			if (oChild.getNodeType() == Node.ELEMENT_NODE) {
				if (mLogger.isDebugEnabled()) mLogger.debug("Reading node: " + oChild.toString());
				if (Constants.ELEMENT_ERRORHANDLER.equals(oChild.getNodeName())) {
					readErrorHandler(oChild, oConfig);
				} else if (Constants.ELEMENT_REGISTER.equals(oChild.getNodeName())) {
					if (oChild.getAttributes().getNamedItem(Constants.ATTR_PARSER) != null) {
						readRegisterParser(oChild, oConfig);
					} else if (oChild.getAttributes().getNamedItem(Constants.ATTR_TRANSFORM) != null) {
						readRegisterTransform(oChild, oConfig);
					} else if (oChild.getAttributes().getNamedItem(Constants.ATTR_LOGGER) != null) {
						readRegisterLogger(oChild, oConfig);
					}
				} else if (Constants.ELEMENT_MAPPING.equals(oChild.getNodeName())) {
					readMapping(oChild, oConfig);
				} else {
					readRule(oChild, oConfig);
				}
			}
		}
		
		return oConfig;
	}
	
	private void readRegisterTransform(Node pNode, RewriterConfiguration pConfig) throws ConfigurationException {
		Node oClassNode = ParserHelper.getAttribute(pNode, Constants.ATTR_TRANSFORM, true);
		if (pNode.getChildNodes() != null && pNode.getChildNodes().getLength() > 0) {
			throw new ConfigurationException(String.format(Messages.ELEMENT_NOELEMENTS, Constants.ELEMENT_REGISTER));
		}
		try {
			pConfig.getTransformFactory().addTransform(oClassNode.getNodeValue());
		} catch (Exception e) {
			throw new ConfigurationException("Error creating transform \"" + oClassNode.getNodeValue() + "\".", e);
		}
	}
	
	private void readRegisterLogger(Node pNode, RewriterConfiguration pConfig) throws ConfigurationException {
		Node oClassNode = ParserHelper.getAttribute(pNode, Constants.ATTR_LOGGER, true);
		if (pNode.getChildNodes().getLength() > 0) {
			throw new ConfigurationException(String.format(Messages.ELEMENT_NOELEMENTS, Constants.ELEMENT_REGISTER));
		}
		try {
			Class oClass = Class.forName(oClassNode.getNodeValue());
			IRewriteLogger oLogger = (IRewriteLogger) oClass.newInstance();
			pConfig.setLogger(oLogger);
		} catch (Exception e) {
			throw new ConfigurationException("Error creating logger \"" + oClassNode.getNodeValue() + "\".", e);
		}
	}
	
	private void readRegisterParser(Node pNode, RewriterConfiguration pConfig) throws ConfigurationException {
		Node oClassNode = ParserHelper.getAttribute(pNode, Constants.ATTR_PARSER, true);
		if (pNode.getChildNodes().getLength() > 0) {
			throw new ConfigurationException(String.format(Messages.ELEMENT_NOELEMENTS, Constants.ELEMENT_REGISTER));
		}
		try {
			Class oClass = Class.forName(oClassNode.getNodeValue());
			Object oParser = oClass.newInstance();
			if (oParser instanceof IRewriteActionParser) {
				pConfig.getActionParserFactory().addParser((IRewriteActionParser) oParser);
			}
			if (oParser instanceof IRewriteConditionParser) {
				pConfig.getConditionParserPipeline().add((IRewriteConditionParser) oParser);
			}
		} catch (Exception e) {
			throw new ConfigurationException("Error creating parser \"" + oClassNode.getNodeValue() + "\".", e);
		}
	}
	
	private void readErrorHandler(Node pNode, RewriterConfiguration pConfig) throws ConfigurationException {
		Node oCodeNode = ParserHelper.getAttribute(pNode, Constants.ATTR_CODE, true);
		Node oClassNode = pNode.getAttributes().getNamedItem(Constants.ATTR_TYPE);
		Node oUrlNode = pNode.getAttributes().getNamedItem(Constants.ATTR_URL);
		if (oClassNode == null && oUrlNode == null) {
			throw new ConfigurationException(String.format(Messages.ATTRIBUTE_REQUIRED, Constants.ATTR_URL));
		}
		IRewriteErrorHandler oHandler = null;
		try {
			if (oClassNode != null) {
				Class oClass = Class.forName(oClassNode.getNodeValue());
				oHandler = (IRewriteErrorHandler) oClass.newInstance();
			} else {
				oHandler = new DefaultErrorHandler(oUrlNode.getNodeValue());
			}
		} catch (Exception e) {
			throw new ConfigurationException(String.format(Messages.INVALID_TYPE, new Object[] {oClassNode.getNodeValue(), IRewriteErrorHandler.class.getName()}));			
		}
		pConfig.getErrorHandlers().put(Integer.parseInt(oCodeNode.getNodeValue()), oHandler);
	}
	
	private void readMapping(Node pNode, RewriterConfiguration pConfig) throws ConfigurationException {
		Node oNameNode = ParserHelper.getAttribute(pNode, Constants.ATTR_NAME, true); 
		StringDictionnary oMap = new StringDictionnary();
		for (int i = 0; i < pNode.getChildNodes().getLength(); i++) {
			Node oNode = pNode.getChildNodes().item(i);
			if (Constants.ELEMENT_MAP.equals(oNode.getNodeName()) && oNode.getNodeType() == Node.ELEMENT_NODE) {
				Node oFromValueNode = ParserHelper.getAttribute(oNode, Constants.ATTR_FROM, true);
				Node oToValueNode =  ParserHelper.getAttribute(oNode, Constants.ATTR_TO, true);
				oMap.put(oFromValueNode.getNodeValue(), oToValueNode.getNodeValue());
			} else if (oNode.getNodeType() != Node.TEXT_NODE){
				throw new ConfigurationException(String.format(Messages.ELEMENT_NOTALLOWED, oNode.getNodeName()));
			}
		}
		pConfig.getTransformFactory().addTransform(new StaticMappingTransform(oNameNode.getNodeValue(), oMap));
	}
	
	private void readRule(Node pNode, RewriterConfiguration pConfig) throws ConfigurationException {
		if (mLogger.isDebugEnabled()) mLogger.debug("Reading rule with node " + pNode.toString());
		boolean oParsed = false;
		List<IRewriteActionParser> oParsers = pConfig.getActionParserFactory().getParsers(pNode.getNodeName());
		if (oParsers != null) {
			for (IRewriteActionParser oParser : oParsers) {
				if (!oParser.getAllowNestedActions() && pNode.getChildNodes().getLength() > 0) {
					throw new ConfigurationException(String.format(Messages.ELEMENT_NOELEMENTS, oParser.getName()));
				}
				if (!oParser.getAllowAttributes() && pNode.getAttributes().getLength() > 0) {
					throw new ConfigurationException(String.format(Messages.ELEMENT_NOATTRIBUTES, oParser.getName()));
				}
				if (mLogger.isDebugEnabled()) mLogger.debug(oParser.getClass().getName() + " parsing node...");
				IRewriteAction oRule = oParser.parse(pNode, pConfig);
				if (oRule != null) {
					pConfig.getRules().add(oRule);
					oParsed = true;
					break;
				}
			}
		}
		if (!oParsed) {
			throw new ConfigurationException(String.format(Messages.ELEMENT_NOTALLOWED, pNode.getNodeName()));
		}
	}
	

}
